"use strict";

function once(func) {
    var called = false;
    return function() {
        if (called) return;
        called = true;
        func.apply(this, arguments);
        func = null;
    };
};

module.exports = function(fn, noqueue) {

    var lock = false;
    var queue = [];
    var _this = null;

    function _process() {

        if(queue.length === 0 || lock === true) return;

        lock = true;

        var data = queue.shift();

        var cb = once(function(){

            lock = false;

            if(data.cb) {
                data.cb.apply(null, arguments);
            }

            process.nextTick(_process);
        })

        fn.apply(_this, data.args.concat([cb]));
    }

    return function(){

        _this = this; // so that we can be used as object prototype method

        if(noqueue && lock === true) { // non queued mode
            return;
        }

        var data = {
            args: Array.prototype.slice.call(arguments),
            cb: null
        }

        if(typeof arguments[arguments.length - 1] === 'function'){
            data.cb = arguments[arguments.length - 1];
            data.args = data.args.slice(0, -1);
        }

        queue.push(data)

        if(lock === false){
            process.nextTick(_process);
        }
    }
}